@@ -1,6 +1,8 @@
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 import pytest
 
+
 xfail = pytest.mark.xfail
 
 
