@@ -1,21 +1,21 @@
+# mypy: allow-untyped-defs
+# ruff: noqa: T100
 """Interactive debugging with PDB, the Python Debugger."""
+
+from __future__ import annotations
+
 import argparse
 import functools
 import sys
 import types
-import unittest
 from typing import Any
 from typing import Callable
 from typing import Generator
-from typing import List
-from typing import Optional
-from typing import Tuple
-from typing import Type
-from typing import TYPE_CHECKING
-from typing import Union
+import unittest
 
 from _pytest import outcomes
 from _pytest._code import ExceptionInfo
+from _pytest.capture import CaptureManager
 from _pytest.config import Config
 from _pytest.config import ConftestImportFailure
 from _pytest.config import hookimpl
@@ -24,13 +24,10 @@ from _pytest.config.argparsing import Parser
 from _pytest.config.exceptions import UsageError
 from _pytest.nodes import Node
 from _pytest.reports import BaseReport
-
-if TYPE_CHECKING:
-    from _pytest.capture import CaptureManager
-    from _pytest.runner import CallInfo
+from _pytest.runner import CallInfo
 
 
-def _validate_usepdb_cls(value: str) -> Tuple[str, str]:
+def _validate_usepdb_cls(value: str) -> tuple[str, str]:
     """Validate syntax of --pdbcls option."""
     try:
         modname, classname = value.split(":")
@@ -95,22 +92,22 @@ def pytest_configure(config: Config) -> None:
 class pytestPDB:
     """Pseudo PDB that defers to the real pdb."""
 
-    _pluginmanager: Optional[PytestPluginManager] = None
-    _config: Optional[Config] = None
-    _saved: List[
-        Tuple[Callable[..., None], Optional[PytestPluginManager], Optional[Config]]
+    _pluginmanager: PytestPluginManager | None = None
+    _config: Config | None = None
+    _saved: list[
+        tuple[Callable[..., None], PytestPluginManager | None, Config | None]
     ] = []
     _recursive_debug = 0
-    _wrapped_pdb_cls: Optional[Tuple[Type[Any], Type[Any]]] = None
+    _wrapped_pdb_cls: tuple[type[Any], type[Any]] | None = None
 
     @classmethod
-    def _is_capturing(cls, capman: Optional["CaptureManager"]) -> Union[str, bool]:
+    def _is_capturing(cls, capman: CaptureManager | None) -> str | bool:
         if capman:
             return capman.is_capturing()
         return False
 
     @classmethod
-    def _import_pdb_cls(cls, capman: Optional["CaptureManager"]):
+    def _import_pdb_cls(cls, capman: CaptureManager | None):
         if not cls._config:
             import pdb
 
@@ -149,12 +146,10 @@ class pytestPDB:
         return wrapped_cls
 
     @classmethod
-    def _get_pdb_wrapper_class(cls, pdb_cls, capman: Optional["CaptureManager"]):
+    def _get_pdb_wrapper_class(cls, pdb_cls, capman: CaptureManager | None):
         import _pytest.config
 
-        # Type ignored because mypy doesn't support "dynamic"
-        # inheritance like this.
-        class PytestPdbWrapper(pdb_cls):  # type: ignore[valid-type,misc]
+        class PytestPdbWrapper(pdb_cls):
             _pytest_capman = capman
             _continued = False
 
@@ -179,8 +174,7 @@ class pytestPDB:
                         else:
                             tw.sep(
                                 ">",
-                                "PDB continue (IO-capturing resumed for %s)"
-                                % capturing,
+                                f"PDB continue (IO-capturing resumed for {capturing})",
                             )
                         assert capman is not None
                         capman.resume()
@@ -241,7 +235,7 @@ class pytestPDB:
         import _pytest.config
 
         if cls._pluginmanager is None:
-            capman: Optional[CaptureManager] = None
+            capman: CaptureManager | None = None
         else:
             capman = cls._pluginmanager.getplugin("capturemanager")
         if capman:
@@ -263,8 +257,7 @@ class pytestPDB:
                     elif capturing:
                         tw.sep(
                             ">",
-                            "PDB %s (IO-capturing turned off for %s)"
-                            % (method, capturing),
+                            f"PDB {method} (IO-capturing turned off for {capturing})",
                         )
                     else:
                         tw.sep(">", f"PDB {method}")
@@ -285,7 +278,7 @@ class pytestPDB:
 
 class PdbInvoke:
     def pytest_exception_interact(
-        self, node: Node, call: "CallInfo[Any]", report: BaseReport
+        self, node: Node, call: CallInfo[Any], report: BaseReport
     ) -> None:
         capman = node.config.pluginmanager.getplugin("capturemanager")
         if capman:
@@ -304,13 +297,13 @@ class PdbInvoke:
 
 
 class PdbTrace:
-    @hookimpl(hookwrapper=True)
-    def pytest_pyfunc_call(self, pyfuncitem) -> Generator[None, None, None]:
+    @hookimpl(wrapper=True)
+    def pytest_pyfunc_call(self, pyfuncitem) -> Generator[None, object, object]:
         wrap_pytest_function_for_tracing(pyfuncitem)
-        yield
+        return (yield)
 
 
-def wrap_pytest_function_for_tracing(pyfuncitem):
+def wrap_pytest_function_for_tracing(pyfuncitem) -> None:
     """Change the Python function object of the given Function item by a
     wrapper which actually enters pdb before calling the python function
     itself, effectively leaving the user in the pdb prompt in the first
@@ -322,14 +315,14 @@ def wrap_pytest_function_for_tracing(pyfuncitem):
     # python < 3.7.4) runcall's first param is `func`, which means we'd get
     # an exception if one of the kwargs to testfunction was called `func`.
     @functools.wraps(testfunction)
-    def wrapper(*args, **kwargs):
+    def wrapper(*args, **kwargs) -> None:
         func = functools.partial(testfunction, *args, **kwargs)
         _pdb.runcall(func)
 
     pyfuncitem.obj = wrapper
 
 
-def maybe_wrap_pytest_function_for_tracing(pyfuncitem):
+def maybe_wrap_pytest_function_for_tracing(pyfuncitem) -> None:
     """Wrap the given pytestfunct item for tracing support if --trace was given in
     the command line."""
     if pyfuncitem.config.getvalue("trace"):
@@ -339,7 +332,7 @@ def maybe_wrap_pytest_function_for_tracing(pyfuncitem):
 def _enter_pdb(
     node: Node, excinfo: ExceptionInfo[BaseException], rep: BaseReport
 ) -> BaseReport:
-    # XXX we re-use the TerminalReporter's terminalwriter
+    # XXX we reuse the TerminalReporter's terminalwriter
     # because this seems to avoid some encoding related troubles
     # for not completely clear reasons.
     tw = node.config.pluginmanager.getplugin("terminalreporter")._tw
@@ -377,7 +370,8 @@ def _postmortem_traceback(excinfo: ExceptionInfo[BaseException]) -> types.Traceb
     elif isinstance(excinfo.value, ConftestImportFailure):
         # A config.ConftestImportFailure is not useful for post_mortem.
         # Use the underlying exception instead:
-        return excinfo.value.excinfo[2]
+        assert excinfo.value.cause.__traceback__ is not None
+        return excinfo.value.cause.__traceback__
     else:
         assert excinfo._excinfo is not None
         return excinfo._excinfo[2]
