@@ -1,8 +1,11 @@
+# mypy: allow-untyped-defs
 """(Disabled by default) support for testing pytest and pytest plugins.
 
 PYTEST_DONT_REWRITE
 """
 
+from __future__ import annotations
+
 import collections.abc
 import contextlib
 from fnmatch import fnmatch
@@ -20,22 +23,16 @@ import sys
 import traceback
 from typing import Any
 from typing import Callable
-from typing import Dict
 from typing import Final
 from typing import final
 from typing import Generator
 from typing import IO
 from typing import Iterable
-from typing import List
 from typing import Literal
-from typing import Optional
 from typing import overload
 from typing import Sequence
 from typing import TextIO
-from typing import Tuple
-from typing import Type
 from typing import TYPE_CHECKING
-from typing import Union
 from weakref import WeakKeyDictionary
 
 from iniconfig import IniConfig
@@ -122,7 +119,7 @@ def pytest_configure(config: Config) -> None:
 
 
 class LsofFdLeakChecker:
-    def get_open_files(self) -> List[Tuple[str, str]]:
+    def get_open_files(self) -> list[tuple[str, str]]:
         if sys.version_info >= (3, 11):
             # New in Python 3.11, ignores utf-8 mode
             encoding = locale.getencoding()
@@ -181,13 +178,13 @@ class LsofFdLeakChecker:
             leaked_files = [t for t in lines2 if t[0] in new_fds]
             if leaked_files:
                 error = [
-                    "***** %s FD leakage detected" % len(leaked_files),
+                    f"***** {len(leaked_files)} FD leakage detected",
                     *(str(f) for f in leaked_files),
                     "*** Before:",
                     *(str(f) for f in lines1),
                     "*** After:",
                     *(str(f) for f in lines2),
-                    "***** %s FD leakage detected" % len(leaked_files),
+                    f"***** {len(leaked_files)} FD leakage detected",
                     "*** function {}:{}: {} ".format(*item.location),
                     "See issue #2366",
                 ]
@@ -198,7 +195,7 @@ class LsofFdLeakChecker:
 
 
 @fixture
-def _pytest(request: FixtureRequest) -> "PytestArg":
+def _pytest(request: FixtureRequest) -> PytestArg:
     """Return a helper which offers a gethookrecorder(hook) method which
     returns a HookRecorder instance which helps to make assertions about called
     hooks."""
@@ -209,13 +206,13 @@ class PytestArg:
     def __init__(self, request: FixtureRequest) -> None:
         self._request = request
 
-    def gethookrecorder(self, hook) -> "HookRecorder":
+    def gethookrecorder(self, hook) -> HookRecorder:
         hookrecorder = HookRecorder(hook._pm)
         self._request.addfinalizer(hookrecorder.finish_recording)
         return hookrecorder
 
 
-def get_public_names(values: Iterable[str]) -> List[str]:
+def get_public_names(values: Iterable[str]) -> list[str]:
     """Only return names from iterator values without a leading underscore."""
     return [x for x in values if x[0] != "_"]
 
@@ -245,8 +242,7 @@ class RecordedHookCall:
 
     if TYPE_CHECKING:
         # The class has undetermined attributes, this tells mypy about it.
-        def __getattr__(self, key: str):
-            ...
+        def __getattr__(self, key: str): ...
 
 
 @final
@@ -265,8 +261,8 @@ class HookRecorder:
         check_ispytest(_ispytest)
 
         self._pluginmanager = pluginmanager
-        self.calls: List[RecordedHookCall] = []
-        self.ret: Optional[Union[int, ExitCode]] = None
+        self.calls: list[RecordedHookCall] = []
+        self.ret: int | ExitCode | None = None
 
         def before(hook_name: str, hook_impls, kwargs) -> None:
             self.calls.append(RecordedHookCall(hook_name, kwargs))
@@ -279,17 +275,18 @@ class HookRecorder:
     def finish_recording(self) -> None:
         self._undo_wrapping()
 
-    def getcalls(self, names: Union[str, Iterable[str]]) -> List[RecordedHookCall]:
+    def getcalls(self, names: str | Iterable[str]) -> list[RecordedHookCall]:
         """Get all recorded calls to hooks with the given names (or name)."""
         if isinstance(names, str):
             names = names.split()
         return [call for call in self.calls if call._name in names]
 
-    def assert_contains(self, entries: Sequence[Tuple[str, str]]) -> None:
+    def assert_contains(self, entries: Sequence[tuple[str, str]]) -> None:
         __tracebackhide__ = True
         i = 0
         entries = list(entries)
-        backlocals = sys._getframe(1).f_locals
+        # Since Python 3.13, f_locals is not a dict, but eval requires a dict.
+        backlocals = dict(sys._getframe(1).f_locals)
         while entries:
             name, check = entries.pop(0)
             for ind, call in enumerate(self.calls[i:]):
@@ -313,7 +310,7 @@ class HookRecorder:
                 del self.calls[i]
                 return call
         lines = [f"could not find call {name!r}, in:"]
-        lines.extend(["  %s" % x for x in self.calls])
+        lines.extend([f"  {x}" for x in self.calls])
         fail("\n".join(lines))
 
     def getcall(self, name: str) -> RecordedHookCall:
@@ -326,45 +323,42 @@ class HookRecorder:
     @overload
     def getreports(
         self,
-        names: "Literal['pytest_collectreport']",
-    ) -> Sequence[CollectReport]:
-        ...
+        names: Literal["pytest_collectreport"],
+    ) -> Sequence[CollectReport]: ...
 
     @overload
     def getreports(
         self,
-        names: "Literal['pytest_runtest_logreport']",
-    ) -> Sequence[TestReport]:
-        ...
+        names: Literal["pytest_runtest_logreport"],
+    ) -> Sequence[TestReport]: ...
 
     @overload
     def getreports(
         self,
-        names: Union[str, Iterable[str]] = (
+        names: str | Iterable[str] = (
             "pytest_collectreport",
             "pytest_runtest_logreport",
         ),
-    ) -> Sequence[Union[CollectReport, TestReport]]:
-        ...
+    ) -> Sequence[CollectReport | TestReport]: ...
 
     def getreports(
         self,
-        names: Union[str, Iterable[str]] = (
+        names: str | Iterable[str] = (
             "pytest_collectreport",
             "pytest_runtest_logreport",
         ),
-    ) -> Sequence[Union[CollectReport, TestReport]]:
+    ) -> Sequence[CollectReport | TestReport]:
         return [x.report for x in self.getcalls(names)]
 
     def matchreport(
         self,
         inamepart: str = "",
-        names: Union[str, Iterable[str]] = (
+        names: str | Iterable[str] = (
             "pytest_runtest_logreport",
             "pytest_collectreport",
         ),
-        when: Optional[str] = None,
-    ) -> Union[CollectReport, TestReport]:
+        when: str | None = None,
+    ) -> CollectReport | TestReport:
         """Return a testreport whose dotted import path matches."""
         values = []
         for rep in self.getreports(names=names):
@@ -389,34 +383,31 @@ class HookRecorder:
     @overload
     def getfailures(
         self,
-        names: "Literal['pytest_collectreport']",
-    ) -> Sequence[CollectReport]:
-        ...
+        names: Literal["pytest_collectreport"],
+    ) -> Sequence[CollectReport]: ...
 
     @overload
     def getfailures(
         self,
-        names: "Literal['pytest_runtest_logreport']",
-    ) -> Sequence[TestReport]:
-        ...
+        names: Literal["pytest_runtest_logreport"],
+    ) -> Sequence[TestReport]: ...
 
     @overload
     def getfailures(
         self,
-        names: Union[str, Iterable[str]] = (
+        names: str | Iterable[str] = (
             "pytest_collectreport",
             "pytest_runtest_logreport",
         ),
-    ) -> Sequence[Union[CollectReport, TestReport]]:
-        ...
+    ) -> Sequence[CollectReport | TestReport]: ...
 
     def getfailures(
         self,
-        names: Union[str, Iterable[str]] = (
+        names: str | Iterable[str] = (
             "pytest_collectreport",
             "pytest_runtest_logreport",
         ),
-    ) -> Sequence[Union[CollectReport, TestReport]]:
+    ) -> Sequence[CollectReport | TestReport]:
         return [rep for rep in self.getreports(names) if rep.failed]
 
     def getfailedcollections(self) -> Sequence[CollectReport]:
@@ -424,10 +415,10 @@ class HookRecorder:
 
     def listoutcomes(
         self,
-    ) -> Tuple[
+    ) -> tuple[
         Sequence[TestReport],
-        Sequence[Union[CollectReport, TestReport]],
-        Sequence[Union[CollectReport, TestReport]],
+        Sequence[CollectReport | TestReport],
+        Sequence[CollectReport | TestReport],
     ]:
         passed = []
         skipped = []
@@ -446,7 +437,7 @@ class HookRecorder:
                 failed.append(rep)
         return passed, skipped, failed
 
-    def countoutcomes(self) -> List[int]:
+    def countoutcomes(self) -> list[int]:
         return [len(x) for x in self.listoutcomes()]
 
     def assertoutcome(self, passed: int = 0, skipped: int = 0, failed: int = 0) -> None:
@@ -466,14 +457,14 @@ class HookRecorder:
 
 
 @fixture
-def linecomp() -> "LineComp":
+def linecomp() -> LineComp:
     """A :class: `LineComp` instance for checking that an input linearly
     contains a sequence of strings."""
     return LineComp()
 
 
 @fixture(name="LineMatcher")
-def LineMatcher_fixture(request: FixtureRequest) -> Type["LineMatcher"]:
+def LineMatcher_fixture(request: FixtureRequest) -> type[LineMatcher]:
     """A reference to the :class: `LineMatcher`.
 
     This is instantiable with a list of lines (without their trailing newlines).
@@ -485,7 +476,7 @@ def LineMatcher_fixture(request: FixtureRequest) -> Type["LineMatcher"]:
 @fixture
 def pytester(
     request: FixtureRequest, tmp_path_factory: TempPathFactory, monkeypatch: MonkeyPatch
-) -> "Pytester":
+) -> Pytester:
     """
     Facilities to write tests/configuration files, execute pytest in isolation, and match
     against expected output, perfect for black-box testing of pytest plugins.
@@ -500,7 +491,7 @@ def pytester(
 
 
 @fixture
-def _sys_snapshot() -> Generator[None, None, None]:
+def _sys_snapshot() -> Generator[None]:
     snappaths = SysPathsSnapshot()
     snapmods = SysModulesSnapshot()
     yield
@@ -509,7 +500,7 @@ def _sys_snapshot() -> Generator[None, None, None]:
 
 
 @fixture
-def _config_for_test() -> Generator[Config, None, None]:
+def _config_for_test() -> Generator[Config]:
     from _pytest.config import get_config
 
     config = get_config()
@@ -529,13 +520,13 @@ class RunResult:
 
     def __init__(
         self,
-        ret: Union[int, ExitCode],
-        outlines: List[str],
-        errlines: List[str],
+        ret: int | ExitCode,
+        outlines: list[str],
+        errlines: list[str],
         duration: float,
     ) -> None:
         try:
-            self.ret: Union[int, ExitCode] = ExitCode(ret)
+            self.ret: int | ExitCode = ExitCode(ret)
             """The return value."""
         except ValueError:
             self.ret = ret
@@ -560,7 +551,7 @@ class RunResult:
             % (self.ret, len(self.stdout.lines), len(self.stderr.lines), self.duration)
         )
 
-    def parseoutcomes(self) -> Dict[str, int]:
+    def parseoutcomes(self) -> dict[str, int]:
         """Return a dictionary of outcome noun -> count from parsing the terminal
         output that the test process produced.
 
@@ -573,7 +564,7 @@ class RunResult:
         return self.parse_summary_nouns(self.outlines)
 
     @classmethod
-    def parse_summary_nouns(cls, lines) -> Dict[str, int]:
+    def parse_summary_nouns(cls, lines) -> dict[str, int]:
         """Extract the nouns from a pytest terminal summary line.
 
         It always returns the plural noun for consistency::
@@ -604,8 +595,8 @@ class RunResult:
         errors: int = 0,
         xpassed: int = 0,
         xfailed: int = 0,
-        warnings: Optional[int] = None,
-        deselected: Optional[int] = None,
+        warnings: int | None = None,
+        deselected: int | None = None,
     ) -> None:
         """
         Assert that the specified outcomes appear with the respective
@@ -631,7 +622,7 @@ class RunResult:
 
 
 class SysModulesSnapshot:
-    def __init__(self, preserve: Optional[Callable[[str], bool]] = None) -> None:
+    def __init__(self, preserve: Callable[[str], bool] | None = None) -> None:
         self.__preserve = preserve
         self.__saved = dict(sys.modules)
 
@@ -664,7 +655,7 @@ class Pytester:
 
     __test__ = False
 
-    CLOSE_STDIN: "Final" = NOTSET
+    CLOSE_STDIN: Final = NOTSET
 
     class TimeoutExpired(Exception):
         pass
@@ -679,9 +670,9 @@ class Pytester:
     ) -> None:
         check_ispytest(_ispytest)
         self._request = request
-        self._mod_collections: WeakKeyDictionary[
-            Collector, List[Union[Item, Collector]]
-        ] = WeakKeyDictionary()
+        self._mod_collections: WeakKeyDictionary[Collector, list[Item | Collector]] = (
+            WeakKeyDictionary()
+        )
         if request.function:
             name: str = request.function.__name__
         else:
@@ -692,7 +683,7 @@ class Pytester:
         #: :py:meth:`runpytest`.  Initially this is an empty list but plugins can
         #: be added to the list.  The type of items to add to the list depends on
         #: the method using them so refer to them for details.
-        self.plugins: List[Union[str, _PluggyPlugin]] = []
+        self.plugins: list[str | _PluggyPlugin] = []
         self._sys_path_snapshot = SysPathsSnapshot()
         self._sys_modules_snapshot = self.__take_sys_modules_snapshot()
         self._request.addfinalizer(self._finalize)
@@ -760,18 +751,21 @@ class Pytester:
     def _makefile(
         self,
         ext: str,
-        lines: Sequence[Union[Any, bytes]],
-        files: Dict[str, str],
+        lines: Sequence[Any | bytes],
+        files: dict[str, str],
         encoding: str = "utf-8",
     ) -> Path:
         items = list(files.items())
 
+        if ext is None:
+            raise TypeError("ext must not be None")
+
         if ext and not ext.startswith("."):
             raise ValueError(
                 f"pytester.makefile expects a file extension, try .{ext} instead of {ext}"
             )
 
-        def to_text(s: Union[Any, bytes]) -> str:
+        def to_text(s: Any | bytes) -> str:
             return s.decode(encoding) if isinstance(s, bytes) else str(s)
 
         if lines:
@@ -807,6 +801,7 @@ class Pytester:
             The first created file.
 
         Examples:
+
         .. code-block:: python
 
             pytester.makefile(".txt", "line1", "line2")
@@ -860,6 +855,7 @@ class Pytester:
         existing files.
 
         Examples:
+
         .. code-block:: python
 
             def test_something(pytester):
@@ -879,6 +875,7 @@ class Pytester:
         existing files.
 
         Examples:
+
         .. code-block:: python
 
             def test_something(pytester):
@@ -891,9 +888,7 @@ class Pytester:
         """
         return self._makefile(".txt", args, kwargs)
 
-    def syspathinsert(
-        self, path: Optional[Union[str, "os.PathLike[str]"]] = None
-    ) -> None:
+    def syspathinsert(self, path: str | os.PathLike[str] | None = None) -> None:
         """Prepend a directory to sys.path, defaults to :attr:`path`.
 
         This is undone automatically when this object dies at the end of each
@@ -907,19 +902,20 @@ class Pytester:
 
         self._monkeypatch.syspath_prepend(str(path))
 
-    def mkdir(self, name: Union[str, "os.PathLike[str]"]) -> Path:
+    def mkdir(self, name: str | os.PathLike[str]) -> Path:
         """Create a new (sub)directory.
 
         :param name:
             The name of the directory, relative to the pytester path.
         :returns:
             The created directory.
+        :rtype: pathlib.Path
         """
         p = self.path / name
         p.mkdir()
         return p
 
-    def mkpydir(self, name: Union[str, "os.PathLike[str]"]) -> Path:
+    def mkpydir(self, name: str | os.PathLike[str]) -> Path:
         """Create a new python package.
 
         This creates a (sub)directory with an empty ``__init__.py`` file so it
@@ -930,13 +926,14 @@ class Pytester:
         p.joinpath("__init__.py").touch()
         return p
 
-    def copy_example(self, name: Optional[str] = None) -> Path:
+    def copy_example(self, name: str | None = None) -> Path:
         """Copy file from project's directory into the testdir.
 
         :param name:
             The name of the file to copy.
         :return:
             Path to the copied directory (inside ``self.path``).
+        :rtype: pathlib.Path
         """
         example_dir_ = self._request.config.getini("pytester_example_dir")
         if example_dir_ is None:
@@ -975,9 +972,7 @@ class Pytester:
                 f'example "{example_path}" is not found as a file or directory'
             )
 
-    def getnode(
-        self, config: Config, arg: Union[str, "os.PathLike[str]"]
-    ) -> Union[Collector, Item]:
+    def getnode(self, config: Config, arg: str | os.PathLike[str]) -> Collector | Item:
         """Get the collection node of a file.
 
         :param config:
@@ -996,9 +991,7 @@ class Pytester:
         config.hook.pytest_sessionfinish(session=session, exitstatus=ExitCode.OK)
         return res
 
-    def getpathnode(
-        self, path: Union[str, "os.PathLike[str]"]
-    ) -> Union[Collector, Item]:
+    def getpathnode(self, path: str | os.PathLike[str]) -> Collector | Item:
         """Return the collection node of a file.
 
         This is like :py:meth:`getnode` but uses :py:meth:`parseconfigure` to
@@ -1018,7 +1011,7 @@ class Pytester:
         config.hook.pytest_sessionfinish(session=session, exitstatus=ExitCode.OK)
         return res
 
-    def genitems(self, colitems: Sequence[Union[Item, Collector]]) -> List[Item]:
+    def genitems(self, colitems: Sequence[Item | Collector]) -> list[Item]:
         """Generate all test items from a collection node.
 
         This recurses into the collection node and returns a list of all the
@@ -1030,7 +1023,7 @@ class Pytester:
             The collected items.
         """
         session = colitems[0].session
-        result: List[Item] = []
+        result: list[Item] = []
         for colitem in colitems:
             result.extend(session.genitems(colitem))
         return result
@@ -1061,10 +1054,10 @@ class Pytester:
         :param cmdlineargs: Any extra command line arguments to use.
         """
         p = self.makepyfile(source)
-        values = list(cmdlineargs) + [p]
+        values = [*list(cmdlineargs), p]
         return self.inline_run(*values)
 
-    def inline_genitems(self, *args) -> Tuple[List[Item], HookRecorder]:
+    def inline_genitems(self, *args) -> tuple[list[Item], HookRecorder]:
         """Run ``pytest.main(['--collect-only'])`` in-process.
 
         Runs the :py:func:`pytest.main` function to run all of pytest inside
@@ -1077,7 +1070,7 @@ class Pytester:
 
     def inline_run(
         self,
-        *args: Union[str, "os.PathLike[str]"],
+        *args: str | os.PathLike[str],
         plugins=(),
         no_reraise_ctrlc: bool = False,
     ) -> HookRecorder:
@@ -1147,7 +1140,7 @@ class Pytester:
                 finalizer()
 
     def runpytest_inprocess(
-        self, *args: Union[str, "os.PathLike[str]"], **kwargs: Any
+        self, *args: str | os.PathLike[str], **kwargs: Any
     ) -> RunResult:
         """Return result of running pytest in-process, providing a similar
         interface to what self.runpytest() provides."""
@@ -1190,9 +1183,7 @@ class Pytester:
         res.reprec = reprec  # type: ignore
         return res
 
-    def runpytest(
-        self, *args: Union[str, "os.PathLike[str]"], **kwargs: Any
-    ) -> RunResult:
+    def runpytest(self, *args: str | os.PathLike[str], **kwargs: Any) -> RunResult:
         """Run pytest inline or in a subprocess, depending on the command line
         option "--runpytest" and return a :py:class:`~pytest.RunResult`."""
         new_args = self._ensure_basetemp(args)
@@ -1203,17 +1194,19 @@ class Pytester:
         raise RuntimeError(f"Unrecognized runpytest option: {self._method}")
 
     def _ensure_basetemp(
-        self, args: Sequence[Union[str, "os.PathLike[str]"]]
-    ) -> List[Union[str, "os.PathLike[str]"]]:
+        self, args: Sequence[str | os.PathLike[str]]
+    ) -> list[str | os.PathLike[str]]:
         new_args = list(args)
         for x in new_args:
             if str(x).startswith("--basetemp"):
                 break
         else:
-            new_args.append("--basetemp=%s" % self.path.parent.joinpath("basetemp"))
+            new_args.append(
+                "--basetemp={}".format(self.path.parent.joinpath("basetemp"))
+            )
         return new_args
 
-    def parseconfig(self, *args: Union[str, "os.PathLike[str]"]) -> Config:
+    def parseconfig(self, *args: str | os.PathLike[str]) -> Config:
         """Return a new pytest :class:`pytest.Config` instance from given
         commandline args.
 
@@ -1237,7 +1230,7 @@ class Pytester:
         self._request.addfinalizer(config._ensure_unconfigure)
         return config
 
-    def parseconfigure(self, *args: Union[str, "os.PathLike[str]"]) -> Config:
+    def parseconfigure(self, *args: str | os.PathLike[str]) -> Config:
         """Return a new pytest configured Config instance.
 
         Returns a new :py:class:`pytest.Config` instance like
@@ -1249,7 +1242,7 @@ class Pytester:
         return config
 
     def getitem(
-        self, source: Union[str, "os.PathLike[str]"], funcname: str = "test_func"
+        self, source: str | os.PathLike[str], funcname: str = "test_func"
     ) -> Item:
         """Return the test item for a test function.
 
@@ -1270,7 +1263,7 @@ class Pytester:
                 return item
         assert 0, f"{funcname!r} item not found in module:\n{source}\nitems: {items}"
 
-    def getitems(self, source: Union[str, "os.PathLike[str]"]) -> List[Item]:
+    def getitems(self, source: str | os.PathLike[str]) -> list[Item]:
         """Return all test items collected from the module.
 
         Writes the source to a Python file and runs pytest's collection on
@@ -1281,7 +1274,7 @@ class Pytester:
 
     def getmodulecol(
         self,
-        source: Union[str, "os.PathLike[str]"],
+        source: str | os.PathLike[str],
         configargs=(),
         *,
         withinit: bool = False,
@@ -1313,9 +1306,7 @@ class Pytester:
         self.config = config = self.parseconfigure(path, *configargs)
         return self.getnode(config, path)
 
-    def collect_by_name(
-        self, modcol: Collector, name: str
-    ) -> Optional[Union[Item, Collector]]:
+    def collect_by_name(self, modcol: Collector, name: str) -> Item | Collector | None:
         """Return the collection node for name from the module collection.
 
         Searches a module collection node for a collection node matching the
@@ -1333,10 +1324,10 @@ class Pytester:
 
     def popen(
         self,
-        cmdargs: Sequence[Union[str, "os.PathLike[str]"]],
-        stdout: Union[int, TextIO] = subprocess.PIPE,
-        stderr: Union[int, TextIO] = subprocess.PIPE,
-        stdin: Union[NotSetType, bytes, IO[Any], int] = CLOSE_STDIN,
+        cmdargs: Sequence[str | os.PathLike[str]],
+        stdout: int | TextIO = subprocess.PIPE,
+        stderr: int | TextIO = subprocess.PIPE,
+        stdin: NotSetType | bytes | IO[Any] | int = CLOSE_STDIN,
         **kw,
     ):
         """Invoke :py:class:`subprocess.Popen`.
@@ -1371,9 +1362,9 @@ class Pytester:
 
     def run(
         self,
-        *cmdargs: Union[str, "os.PathLike[str]"],
-        timeout: Optional[float] = None,
-        stdin: Union[NotSetType, bytes, IO[Any], int] = CLOSE_STDIN,
+        *cmdargs: str | os.PathLike[str],
+        timeout: float | None = None,
+        stdin: NotSetType | bytes | IO[Any] | int = CLOSE_STDIN,
     ) -> RunResult:
         """Run a command with arguments.
 
@@ -1401,8 +1392,10 @@ class Pytester:
             - Otherwise, it is passed through to :py:class:`subprocess.Popen`.
               For further information in this case, consult the document of the
               ``stdin`` parameter in :py:class:`subprocess.Popen`.
+        :type stdin: _pytest.compat.NotSetType | bytes | IO[Any] | int
         :returns:
             The result.
+
         """
         __tracebackhide__ = True
 
@@ -1459,10 +1452,10 @@ class Pytester:
         except UnicodeEncodeError:
             print(f"couldn't print to {fp} because of encoding")
 
-    def _getpytestargs(self) -> Tuple[str, ...]:
+    def _getpytestargs(self) -> tuple[str, ...]:
         return sys.executable, "-mpytest"
 
-    def runpython(self, script: "os.PathLike[str]") -> RunResult:
+    def runpython(self, script: os.PathLike[str]) -> RunResult:
         """Run a python script using sys.executable as interpreter."""
         return self.run(sys.executable, script)
 
@@ -1471,7 +1464,7 @@ class Pytester:
         return self.run(sys.executable, "-c", command)
 
     def runpytest_subprocess(
-        self, *args: Union[str, "os.PathLike[str]"], timeout: Optional[float] = None
+        self, *args: str | os.PathLike[str], timeout: float | None = None
     ) -> RunResult:
         """Run pytest as a subprocess with given arguments.
 
@@ -1491,16 +1484,14 @@ class Pytester:
         """
         __tracebackhide__ = True
         p = make_numbered_dir(root=self.path, prefix="runpytest-", mode=0o700)
-        args = ("--basetemp=%s" % p,) + args
+        args = (f"--basetemp={p}", *args)
         plugins = [x for x in self.plugins if isinstance(x, str)]
         if plugins:
-            args = ("-p", plugins[0]) + args
+            args = ("-p", plugins[0], *args)
         args = self._getpytestargs() + args
         return self.run(*args, timeout=timeout)
 
-    def spawn_pytest(
-        self, string: str, expect_timeout: float = 10.0
-    ) -> "pexpect.spawn":
+    def spawn_pytest(self, string: str, expect_timeout: float = 10.0) -> pexpect.spawn:
         """Run pytest using pexpect.
 
         This makes sure to use the right pytest and sets up the temporary
@@ -1514,7 +1505,7 @@ class Pytester:
         cmd = f"{invoke} --basetemp={basetemp} {string}"
         return self.spawn(cmd, expect_timeout=expect_timeout)
 
-    def spawn(self, cmd: str, expect_timeout: float = 10.0) -> "pexpect.spawn":
+    def spawn(self, cmd: str, expect_timeout: float = 10.0) -> pexpect.spawn:
         """Run a command using pexpect.
 
         The pexpect child is returned.
@@ -1559,9 +1550,9 @@ class LineMatcher:
     ``text.splitlines()``.
     """
 
-    def __init__(self, lines: List[str]) -> None:
+    def __init__(self, lines: list[str]) -> None:
         self.lines = lines
-        self._log_output: List[str] = []
+        self._log_output: list[str] = []
 
     def __str__(self) -> str:
         """Return the entire original text.
@@ -1571,7 +1562,7 @@ class LineMatcher:
         """
         return "\n".join(self.lines)
 
-    def _getlines(self, lines2: Union[str, Sequence[str], Source]) -> Sequence[str]:
+    def _getlines(self, lines2: str | Sequence[str] | Source) -> Sequence[str]:
         if isinstance(lines2, str):
             lines2 = Source(lines2)
         if isinstance(lines2, Source):
@@ -1599,7 +1590,7 @@ class LineMatcher:
                     self._log("matched: ", repr(line))
                     break
             else:
-                msg = "line %r not found in output" % line
+                msg = f"line {line!r} not found in output"
                 self._log(msg)
                 self._fail(msg)
 
@@ -1611,7 +1602,7 @@ class LineMatcher:
         for i, line in enumerate(self.lines):
             if fnline == line or fnmatch(line, fnline):
                 return self.lines[i + 1 :]
-        raise ValueError("line %r not found in output" % fnline)
+        raise ValueError(f"line {fnline!r} not found in output")
 
     def _log(self, *args) -> None:
         self._log_output.append(" ".join(str(x) for x in args))
@@ -1696,7 +1687,7 @@ class LineMatcher:
                     started = True
                     break
                 elif match_func(nextline, line):
-                    self._log("%s:" % match_nickname, repr(line))
+                    self._log(f"{match_nickname}:", repr(line))
                     self._log(
                         "{:>{width}}".format("with:", width=wnick), repr(nextline)
                     )
