@@ -1,9 +1,13 @@
 # mypy: allow-untyped-defs
+from __future__ import annotations
+
 import dataclasses
+import importlib.metadata
 import re
 import sys
 from typing import Generator
-from typing import List
+
+from packaging.version import Version
 
 from _pytest.monkeypatch import MonkeyPatch
 from _pytest.pytester import Pytester
@@ -45,7 +49,7 @@ def reset_colors(monkeypatch: pytest.MonkeyPatch) -> None:
 
 
 @pytest.hookimpl(wrapper=True, tryfirst=True)
-def pytest_collection_modifyitems(items) -> Generator[None, None, None]:
+def pytest_collection_modifyitems(items) -> Generator[None]:
     """Prefer faster tests.
 
     Use a hook wrapper to do this in the beginning, so e.g. --ff still works
@@ -118,8 +122,8 @@ def tw_mock():
             return text
 
         def get_write_msg(self, idx):
-            flag, msg = self.lines[idx]
-            assert flag == TWMock.WRITE
+            assert self.lines[idx][0] == TWMock.WRITE
+            msg = self.lines[idx][1]
             return msg
 
         fullwidth = 80
@@ -167,6 +171,9 @@ def color_mapping():
 
     Used by tests which check the actual colors output by pytest.
     """
+    # https://github.com/pygments/pygments/commit/d24e272894a56a98b1b718d9ac5fabc20124882a
+    pygments_version = Version(importlib.metadata.version("pygments"))
+    pygments_has_kwspace_hl = pygments_version >= Version("2.19")
 
     class ColorMapping:
         COLORS = {
@@ -179,6 +186,7 @@ def color_mapping():
             "bold": "\x1b[1m",
             "reset": "\x1b[0m",
             "kw": "\x1b[94m",
+            "kwspace": "\x1b[90m \x1b[39;49;00m" if pygments_has_kwspace_hl else " ",
             "hl-reset": "\x1b[39;49;00m",
             "function": "\x1b[92m",
             "number": "\x1b[94m",
@@ -190,22 +198,22 @@ def color_mapping():
         NO_COLORS = {k: "" for k in COLORS.keys()}
 
         @classmethod
-        def format(cls, lines: List[str]) -> List[str]:
+        def format(cls, lines: list[str]) -> list[str]:
             """Straightforward replacement of color names to their ASCII codes."""
             return [line.format(**cls.COLORS) for line in lines]
 
         @classmethod
-        def format_for_fnmatch(cls, lines: List[str]) -> List[str]:
+        def format_for_fnmatch(cls, lines: list[str]) -> list[str]:
             """Replace color names for use with LineMatcher.fnmatch_lines"""
             return [line.format(**cls.COLORS).replace("[", "[[]") for line in lines]
 
         @classmethod
-        def format_for_rematch(cls, lines: List[str]) -> List[str]:
+        def format_for_rematch(cls, lines: list[str]) -> list[str]:
             """Replace color names for use with LineMatcher.re_match_lines"""
             return [line.format(**cls.RE_COLORS) for line in lines]
 
         @classmethod
-        def strip_colors(cls, lines: List[str]) -> List[str]:
+        def strip_colors(cls, lines: list[str]) -> list[str]:
             """Entirely remove every color code"""
             return [line.format(**cls.NO_COLORS) for line in lines]
 
