@@ -1,4 +1,6 @@
-# -*- coding: utf-8 -*-
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
 import pytest
 
 
