@@ -1,6 +1,7 @@
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
 import pprint
-from typing import List
-from typing import Tuple
 
 import pytest
 
@@ -15,7 +16,7 @@ def pytest_generate_tests(metafunc):
 
 @pytest.fixture(scope="session")
 def checked_order():
-    order: List[Tuple[str, str, str]] = []
+    order: list[tuple[str, str, str]] = []
 
     yield order
     pprint.pprint(order)
