@@ -1,5 +1,8 @@
+# mypy: allow-untyped-defs
 """setUpModule is always called, even if all tests in the module are skipped"""
 
+from __future__ import annotations
+
 import unittest
 
 
