@@ -1,11 +1,14 @@
 # mypy: disable-error-code="attr-defined"
 # mypy: disallow-untyped-defs
+from __future__ import annotations
+
 import logging
 from typing import Iterator
 
-import pytest
 from _pytest.logging import caplog_records_key
 from _pytest.pytester import Pytester
+import pytest
+
 
 logger = logging.getLogger(__name__)
 sublogger = logging.getLogger(__name__ + ".baz")
@@ -116,7 +119,7 @@ def test_change_disabled_level_undo(pytester: Pytester) -> None:
     result.stdout.no_fnmatch_line("*log from test2*")
 
 
-def test_change_level_undos_handler_level(pytester: Pytester) -> None:
+def test_change_level_undoes_handler_level(pytester: Pytester) -> None:
     """Ensure that 'set_level' is undone after the end of the test (handler).
 
     Issue #7569. Tests the handler level specifically.
@@ -301,7 +304,15 @@ def logging_during_setup_and_teardown(
     assert [x.message for x in caplog.get_records("teardown")] == ["a_teardown_log"]
 
 
-def test_caplog_captures_for_all_stages(
+def private_assert_caplog_records_is_setup_call(
+    caplog: pytest.LogCaptureFixture,
+) -> None:
+    # This reaches into private API, don't use this type of thing in real tests!
+    caplog_records = caplog._item.stash[caplog_records_key]
+    assert set(caplog_records) == {"setup", "call"}
+
+
+def test_captures_for_all_stages(
     caplog: pytest.LogCaptureFixture, logging_during_setup_and_teardown: None
 ) -> None:
     assert not caplog.records
@@ -311,9 +322,7 @@ def test_caplog_captures_for_all_stages(
 
     assert [x.message for x in caplog.get_records("setup")] == ["a_setup_log"]
 
-    # This reaches into private API, don't use this type of thing in real tests!
-    caplog_records = caplog._item.stash[caplog_records_key]
-    assert set(caplog_records) == {"setup", "call"}
+    private_assert_caplog_records_is_setup_call(caplog)
 
 
 def test_clear_for_call_stage(
@@ -322,21 +331,18 @@ def test_clear_for_call_stage(
     logger.info("a_call_log")
     assert [x.message for x in caplog.get_records("call")] == ["a_call_log"]
     assert [x.message for x in caplog.get_records("setup")] == ["a_setup_log"]
-    caplog_records = caplog._item.stash[caplog_records_key]
-    assert set(caplog_records) == {"setup", "call"}
+    private_assert_caplog_records_is_setup_call(caplog)
 
     caplog.clear()
 
     assert caplog.get_records("call") == []
     assert [x.message for x in caplog.get_records("setup")] == ["a_setup_log"]
-    caplog_records = caplog._item.stash[caplog_records_key]
-    assert set(caplog_records) == {"setup", "call"}
+    private_assert_caplog_records_is_setup_call(caplog)
 
     logging.info("a_call_log_after_clear")
     assert [x.message for x in caplog.get_records("call")] == ["a_call_log_after_clear"]
     assert [x.message for x in caplog.get_records("setup")] == ["a_setup_log"]
-    caplog_records = caplog._item.stash[caplog_records_key]
-    assert set(caplog_records) == {"setup", "call"}
+    private_assert_caplog_records_is_setup_call(caplog)
 
 
 def test_ini_controls_global_log_level(pytester: Pytester) -> None:
@@ -362,11 +368,11 @@ def test_ini_controls_global_log_level(pytester: Pytester) -> None:
     )
 
     result = pytester.runpytest()
-    # make sure that that we get a '0' exit code for the testsuite
+    # make sure that we get a '0' exit code for the testsuite
     assert result.ret == 0
 
 
-def test_caplog_can_override_global_log_level(pytester: Pytester) -> None:
+def test_can_override_global_log_level(pytester: Pytester) -> None:
     pytester.makepyfile(
         """
         import pytest
@@ -405,7 +411,7 @@ def test_caplog_can_override_global_log_level(pytester: Pytester) -> None:
     assert result.ret == 0
 
 
-def test_caplog_captures_despite_exception(pytester: Pytester) -> None:
+def test_captures_despite_exception(pytester: Pytester) -> None:
     pytester.makepyfile(
         """
         import pytest
