@@ -1,3 +1,6 @@
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
 import dataclasses
 import itertools
 import re
@@ -7,11 +10,10 @@ from typing import Any
 from typing import cast
 from typing import Dict
 from typing import Iterator
-from typing import List
-from typing import Optional
 from typing import Sequence
-from typing import Tuple
-from typing import Union
+
+import hypothesis
+from hypothesis import strategies
 
 from _pytest import fixtures
 from _pytest import python
@@ -25,17 +27,13 @@ from _pytest.scope import Scope
 import pytest
 
 
-# import hypothesis
-# from hypothesis import strategies
-
-
 class TestMetafunc:
     def Metafunc(self, func, config=None) -> python.Metafunc:
         # The unit tests of this class check if things work correctly
         # on the funcarg level, so we don't need a full blown
         # initialization.
         class FuncFixtureInfoMock:
-            name2fixturedefs: Dict[str, List[fixtures.FixtureDef[object]]] = {}
+            name2fixturedefs: dict[str, list[fixtures.FixtureDef[object]]] = {}
 
             def __init__(self, names):
                 self.names_closure = names
@@ -101,7 +99,7 @@ class TestMetafunc:
             def __repr__(self):
                 return "Exc(from_gen)"
 
-        def gen() -> Iterator[Union[int, None, Exc]]:
+        def gen() -> Iterator[int | None | Exc]:
             yield 0
             yield None
             yield Exc()
@@ -109,7 +107,7 @@ class TestMetafunc:
         metafunc = self.Metafunc(func)
         # When the input is an iterator, only len(args) are taken,
         # so the bad Exc isn't reached.
-        metafunc.parametrize("x", [1, 2], ids=gen())  # type: ignore[arg-type]
+        metafunc.parametrize("x", [1, 2], ids=gen())
         assert [(x.params, x.id) for x in metafunc._calls] == [
             ({"x": 1}, "0"),
             ({"x": 2}, "2"),
@@ -121,7 +119,7 @@ class TestMetafunc:
                 r"Supported types are: .*"
             ),
         ):
-            metafunc.parametrize("x", [1, 2, 3], ids=gen())  # type: ignore[arg-type]
+            metafunc.parametrize("x", [1, 2, 3], ids=gen())
 
     def test_parametrize_bad_scope(self) -> None:
         def func(x):
@@ -293,15 +291,14 @@ class TestMetafunc:
         assert metafunc._calls[2].id == "x1-a"
         assert metafunc._calls[3].id == "x1-b"
 
-    # TODO: Uncomment - https://github.com/HypothesisWorks/hypothesis/pull/3849
-    # @hypothesis.given(strategies.text() | strategies.binary())
-    # @hypothesis.settings(
-    #     deadline=400.0
-    # )  # very close to std deadline and CI boxes are not reliable in CPU power
-    # def test_idval_hypothesis(self, value) -> None:
-    #     escaped = IdMaker([], [], None, None, None, None, None)._idval(value, "a", 6)
-    #     assert isinstance(escaped, str)
-    #     escaped.encode("ascii")
+    @hypothesis.given(strategies.text() | strategies.binary())
+    @hypothesis.settings(
+        deadline=400.0
+    )  # very close to std deadline and CI boxes are not reliable in CPU power
+    def test_idval_hypothesis(self, value) -> None:
+        escaped = IdMaker([], [], None, None, None, None, None)._idval(value, "a", 6)
+        assert isinstance(escaped, str)
+        escaped.encode("ascii")
 
     def test_unicode_idval(self) -> None:
         """Test that Unicode strings outside the ASCII character set get
@@ -347,7 +344,7 @@ class TestMetafunc:
 
         option = "disable_test_id_escaping_and_forfeit_all_rights_to_community_support"
 
-        values: List[Tuple[str, Any, str]] = [
+        values: list[tuple[str, Any, str]] = [
             ("ação", MockConfig({option: True}), "ação"),
             ("ação", MockConfig({option: False}), "a\\xe7\\xe3o"),
         ]
@@ -517,7 +514,7 @@ class TestMetafunc:
     def test_idmaker_idfn(self) -> None:
         """#351"""
 
-        def ids(val: object) -> Optional[str]:
+        def ids(val: object) -> str | None:
             if isinstance(val, Exception):
                 return repr(val)
             return None
@@ -580,7 +577,7 @@ class TestMetafunc:
 
         option = "disable_test_id_escaping_and_forfeit_all_rights_to_community_support"
 
-        values: List[Tuple[Any, str]] = [
+        values: list[tuple[Any, str]] = [
             (MockConfig({option: True}), "ação"),
             (MockConfig({option: False}), "a\\xe7\\xe3o"),
         ]
@@ -618,7 +615,7 @@ class TestMetafunc:
 
         option = "disable_test_id_escaping_and_forfeit_all_rights_to_community_support"
 
-        values: List[Tuple[Any, str]] = [
+        values: list[tuple[Any, str]] = [
             (MockConfig({option: True}), "ação"),
             (MockConfig({option: False}), "a\\xe7\\xe3o"),
         ]
@@ -1749,9 +1746,9 @@ class TestMetafuncFunctionalAuto:
         self, pytester: Pytester, monkeypatch
     ) -> None:
         """Integration test for (#3941)"""
-        class_fix_setup: List[object] = []
+        class_fix_setup: list[object] = []
         monkeypatch.setattr(sys, "class_fix_setup", class_fix_setup, raising=False)
-        func_fix_setup: List[object] = []
+        func_fix_setup: list[object] = []
         monkeypatch.setattr(sys, "func_fix_setup", func_fix_setup, raising=False)
 
         pytester.makepyfile(
