@@ -1,3 +1,6 @@
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
 import re
 import sys
 
@@ -129,6 +132,26 @@ class TestRaises:
         result = pytester.runpytest()
         result.stdout.fnmatch_lines(["*2 failed*"])
 
+    def test_raises_with_invalid_regex(self, pytester: Pytester) -> None:
+        pytester.makepyfile(
+            """
+            import pytest
+
+            def test_invalid_regex():
+                with pytest.raises(ValueError, match="invalid regex character ["):
+                    raise ValueError()
+            """
+        )
+        result = pytester.runpytest()
+        result.stdout.fnmatch_lines(
+            [
+                "*Invalid regex pattern provided to 'match': unterminated character set at position 24*",
+            ]
+        )
+        result.stdout.no_fnmatch_line("*Traceback*")
+        result.stdout.no_fnmatch_line("*File*")
+        result.stdout.no_fnmatch_line("*line*")
+
     def test_noclass(self) -> None:
         with pytest.raises(TypeError):
             pytest.raises("wrong", lambda: None)  # type: ignore[call-overload]
@@ -146,7 +169,7 @@ class TestRaises:
         try:
             pytest.raises(ValueError, int, "0")
         except pytest.fail.Exception as e:
-            assert e.msg == f"DID NOT RAISE {repr(ValueError)}"
+            assert e.msg == f"DID NOT RAISE {ValueError!r}"
         else:
             assert False, "Expected pytest.raises.Exception"
 
@@ -154,7 +177,7 @@ class TestRaises:
             with pytest.raises(ValueError):
                 pass
         except pytest.fail.Exception as e:
-            assert e.msg == f"DID NOT RAISE {repr(ValueError)}"
+            assert e.msg == f"DID NOT RAISE {ValueError!r}"
         else:
             assert False, "Expected pytest.raises.Exception"
 
@@ -279,7 +302,7 @@ class TestRaises:
 
     def test_raises_context_manager_with_kwargs(self):
         with pytest.raises(TypeError) as excinfo:
-            with pytest.raises(Exception, foo="bar"):  # type: ignore[call-overload]
+            with pytest.raises(OSError, foo="bar"):  # type: ignore[call-overload]
                 pass
         assert "Unexpected keyword arguments" in str(excinfo.value)
 
