@@ -1,7 +1,11 @@
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
 import contextlib
 import io
 from io import UnsupportedOperation
 import os
+import re
 import subprocess
 import sys
 import textwrap
@@ -102,16 +106,15 @@ class TestCaptureManager:
 def test_capturing_unicode(pytester: Pytester, method: str) -> None:
     obj = "'b\u00f6y'"
     pytester.makepyfile(
-        """\
+        f"""\
         # taken from issue 227 from nosetests
         def test_unicode():
             import sys
             print(sys.stdout)
-            print(%s)
+            print({obj})
         """
-        % obj
     )
-    result = pytester.runpytest("--capture=%s" % method)
+    result = pytester.runpytest(f"--capture={method}")
     result.stdout.fnmatch_lines(["*1 passed*"])
 
 
@@ -123,7 +126,7 @@ def test_capturing_bytes_in_utf8_encoding(pytester: Pytester, method: str) -> No
             print('b\\u00f6y')
         """
     )
-    result = pytester.runpytest("--capture=%s" % method)
+    result = pytester.runpytest(f"--capture={method}")
     result.stdout.fnmatch_lines(["*1 passed*"])
 
 
@@ -937,7 +940,7 @@ def test_captureresult() -> None:
 
 
 @pytest.fixture
-def tmpfile(pytester: Pytester) -> Generator[BinaryIO, None, None]:
+def tmpfile(pytester: Pytester) -> Generator[BinaryIO]:
     f = pytester.makepyfile("").open("wb+")
     yield f
     if not f.closed:
@@ -1664,3 +1667,32 @@ def test_logging_while_collecting(pytester: Pytester) -> None:
     )
     result.stdout.no_fnmatch_line("*Captured stderr call*")
     result.stdout.no_fnmatch_line("*during collection*")
+
+
+def test_libedit_workaround(pytester: Pytester) -> None:
+    pytester.makeconftest("""
+    import pytest
+
+
+    def pytest_terminal_summary(config):
+        capture = config.pluginmanager.getplugin("capturemanager")
+        capture.suspend_global_capture(in_=True)
+
+        print("Enter 'hi'")
+        value = input()
+        print(f"value: {value!r}")
+
+        capture.resume_global_capture()
+    """)
+    readline = pytest.importorskip("readline")
+    backend = getattr(readline, "backend", readline.__doc__)  # added in Python 3.13
+    print(f"Readline backend: {backend}")
+
+    child = pytester.spawn_pytest("")
+    child.expect(r"Enter 'hi'")
+    child.sendline("hi")
+    rest = child.read().decode("utf8")
+    print(rest)
+    match = re.search(r"^value: '(.*)'\r?$", rest, re.MULTILINE)
+    assert match is not None
+    assert match.group(1) == "hi"
