@@ -1,11 +1,14 @@
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
 import os
 from pathlib import Path
+from pathlib import PurePath
 import pprint
 import shutil
 import sys
 import tempfile
 import textwrap
-from typing import List
 
 from _pytest.assertion.util import running_on_ci
 from _pytest.config import ExitCode
@@ -150,20 +153,17 @@ class TestCollectFS:
         assert "test_notfound" not in s
         assert "test_found" in s
 
-    @pytest.mark.parametrize(
-        "fname",
-        (
-            "activate",
-            "activate.csh",
-            "activate.fish",
-            "Activate",
-            "Activate.bat",
-            "Activate.ps1",
-        ),
+    known_environment_types = pytest.mark.parametrize(
+        "env_path",
+        [
+            pytest.param(PurePath("pyvenv.cfg"), id="venv"),
+            pytest.param(PurePath("conda-meta", "history"), id="conda"),
+        ],
     )
-    def test_ignored_virtualenvs(self, pytester: Pytester, fname: str) -> None:
-        bindir = "Scripts" if sys.platform.startswith("win") else "bin"
-        ensure_file(pytester.path / "virtual" / bindir / fname)
+
+    @known_environment_types
+    def test_ignored_virtualenvs(self, pytester: Pytester, env_path: PurePath) -> None:
+        ensure_file(pytester.path / "virtual" / env_path)
         testfile = ensure_file(pytester.path / "virtual" / "test_invenv.py")
         testfile.write_text("def test_hello(): pass", encoding="utf-8")
 
@@ -177,23 +177,12 @@ class TestCollectFS:
         result = pytester.runpytest("virtual")
         assert "test_invenv" in result.stdout.str()
 
-    @pytest.mark.parametrize(
-        "fname",
-        (
-            "activate",
-            "activate.csh",
-            "activate.fish",
-            "Activate",
-            "Activate.bat",
-            "Activate.ps1",
-        ),
-    )
+    @known_environment_types
     def test_ignored_virtualenvs_norecursedirs_precedence(
-        self, pytester: Pytester, fname: str
+        self, pytester: Pytester, env_path
     ) -> None:
-        bindir = "Scripts" if sys.platform.startswith("win") else "bin"
         # norecursedirs takes priority
-        ensure_file(pytester.path / ".virtual" / bindir / fname)
+        ensure_file(pytester.path / ".virtual" / env_path)
         testfile = ensure_file(pytester.path / ".virtual" / "test_invenv.py")
         testfile.write_text("def test_hello(): pass", encoding="utf-8")
         result = pytester.runpytest("--collect-in-virtualenv")
@@ -202,27 +191,14 @@ class TestCollectFS:
         result = pytester.runpytest("--collect-in-virtualenv", ".virtual")
         assert "test_invenv" in result.stdout.str()
 
-    @pytest.mark.parametrize(
-        "fname",
-        (
-            "activate",
-            "activate.csh",
-            "activate.fish",
-            "Activate",
-            "Activate.bat",
-            "Activate.ps1",
-        ),
-    )
-    def test__in_venv(self, pytester: Pytester, fname: str) -> None:
+    @known_environment_types
+    def test__in_venv(self, pytester: Pytester, env_path: PurePath) -> None:
         """Directly test the virtual env detection function"""
-        bindir = "Scripts" if sys.platform.startswith("win") else "bin"
-        # no bin/activate, not a virtualenv
+        # no env path, not a env
         base_path = pytester.mkdir("venv")
         assert _in_venv(base_path) is False
-        # with bin/activate, totally a virtualenv
-        bin_path = base_path.joinpath(bindir)
-        bin_path.mkdir()
-        bin_path.joinpath(fname).touch()
+        # with env path, totally a env
+        ensure_file(base_path.joinpath(env_path))
         assert _in_venv(base_path) is True
 
     def test_custom_norecursedirs(self, pytester: Pytester) -> None:
@@ -274,14 +250,31 @@ class TestCollectFS:
         # collects the tests
         for dirname in ("a", "b", "c"):
             items, reprec = pytester.inline_genitems(tmp_path.joinpath(dirname))
-            assert [x.name for x in items] == ["test_%s" % dirname]
+            assert [x.name for x in items] == [f"test_{dirname}"]
 
         # changing cwd to each subdirectory and running pytest without
         # arguments collects the tests in that directory normally
         for dirname in ("a", "b", "c"):
             monkeypatch.chdir(pytester.path.joinpath(dirname))
             items, reprec = pytester.inline_genitems()
-            assert [x.name for x in items] == ["test_%s" % dirname]
+            assert [x.name for x in items] == [f"test_{dirname}"]
+
+    def test_missing_permissions_on_unselected_directory_doesnt_crash(
+        self, pytester: Pytester
+    ) -> None:
+        """Regression test for #12120."""
+        test = pytester.makepyfile(test="def test(): pass")
+        bad = pytester.mkdir("bad")
+        try:
+            bad.chmod(0)
+
+            result = pytester.runpytest(test)
+        finally:
+            bad.chmod(750)
+            bad.rmdir()
+
+        assert result.ret == ExitCode.OK
+        result.assert_outcomes(passed=1)
 
 
 class TestCollectPluginHookRelay:
@@ -517,7 +510,7 @@ class TestSession:
         assert len(colitems) == 1
         assert colitems[0].path == topdir
 
-    def get_reported_items(self, hookrec: HookRecorder) -> List[Item]:
+    def get_reported_items(self, hookrec: HookRecorder) -> list[Item]:
         """Return pytest.Item instances reported by the pytest_collectreport hook"""
         calls = hookrec.getcalls("pytest_collectreport")
         return [
@@ -536,7 +529,7 @@ class TestSession:
         newid = item.nodeid
         assert newid == id
         pprint.pprint(hookrec.calls)
-        topdir = pytester.path  # noqa
+        topdir = pytester.path  # noqa: F841
         hookrec.assert_contains(
             [
                 ("pytest_collectstart", "collector.path == topdir"),
@@ -571,7 +564,7 @@ class TestSession:
     def test_collect_custom_nodes_multi_id(self, pytester: Pytester) -> None:
         p = pytester.makepyfile("def test_func(): pass")
         pytester.makeconftest(
-            """
+            f"""
             import pytest
             class SpecialItem(pytest.Item):
                 def runtest(self):
@@ -580,10 +573,9 @@ class TestSession:
                 def collect(self):
                     return [SpecialItem.from_parent(name="check", parent=self)]
             def pytest_collect_file(file_path, parent):
-                if file_path.name == %r:
+                if file_path.name == {p.name!r}:
                     return SpecialFile.from_parent(path=file_path, parent=parent)
         """
-            % p.name
         )
         id = p.name
 
@@ -861,7 +853,7 @@ def test_matchnodes_two_collections_same_file(pytester: Pytester) -> None:
     result = pytester.runpytest()
     assert result.ret == 0
     result.stdout.fnmatch_lines(["*2 passed*"])
-    res = pytester.runpytest("%s::item2" % p.name)
+    res = pytester.runpytest(f"{p.name}::item2")
     res.stdout.fnmatch_lines(["*1 passed*"])
 
 
@@ -1443,7 +1435,7 @@ def test_collect_symlink_out_of_tree(pytester: Pytester) -> None:
     symlink_to_sub = out_of_tree.joinpath("symlink_to_sub")
     symlink_or_skip(sub, symlink_to_sub)
     os.chdir(sub)
-    result = pytester.runpytest("-vs", "--rootdir=%s" % sub, symlink_to_sub)
+    result = pytester.runpytest("-vs", f"--rootdir={sub}", symlink_to_sub)
     result.stdout.fnmatch_lines(
         [
             # Should not contain "sub/"!
@@ -1614,7 +1606,7 @@ def test_fscollector_from_parent(pytester: Pytester, request: FixtureRequest) ->
     assert collector.x == 10
 
 
-def test_class_from_parent(pytester: Pytester, request: FixtureRequest) -> None:
+def test_class_from_parent(request: FixtureRequest) -> None:
     """Ensure Class.from_parent can forward custom arguments to the constructor."""
 
     class MyCollector(pytest.Class):
@@ -1764,7 +1756,7 @@ def test_does_not_crash_on_recursive_symlink(pytester: Pytester) -> None:
 
 @pytest.mark.skipif(not sys.platform.startswith("win"), reason="Windows only")
 def test_collect_short_file_windows(pytester: Pytester) -> None:
-    """Reproducer for #11895: short paths not colleced on Windows."""
+    """Reproducer for #11895: short paths not collected on Windows."""
     short_path = tempfile.mkdtemp()
     if "~" not in short_path:  # pragma: no cover
         if running_on_ci():
@@ -1786,3 +1778,103 @@ def test_collect_short_file_windows(pytester: Pytester) -> None:
     test_file.write_text("def test(): pass", encoding="UTF-8")
     result = pytester.runpytest(short_path)
     assert result.parseoutcomes() == {"passed": 1}
+
+
+def test_pyargs_collection_tree(pytester: Pytester, monkeypatch: MonkeyPatch) -> None:
+    """When using `--pyargs`, the collection tree of a pyargs collection
+    argument should only include parents in the import path, not up to confcutdir.
+
+    Regression test for #11904.
+    """
+    site_packages = pytester.path / "venv/lib/site-packages"
+    site_packages.mkdir(parents=True)
+    monkeypatch.syspath_prepend(site_packages)
+    pytester.makepyfile(
+        **{
+            "venv/lib/site-packages/pkg/__init__.py": "",
+            "venv/lib/site-packages/pkg/sub/__init__.py": "",
+            "venv/lib/site-packages/pkg/sub/test_it.py": "def test(): pass",
+        }
+    )
+
+    result = pytester.runpytest("--pyargs", "--collect-only", "pkg.sub.test_it")
+    assert result.ret == ExitCode.OK
+    result.stdout.fnmatch_lines(
+        [
+            "<Package venv/lib/site-packages/pkg>",
+            "  <Package sub>",
+            "    <Module test_it.py>",
+            "      <Function test>",
+        ],
+        consecutive=True,
+    )
+
+    # Now with an unrelated rootdir with unrelated files.
+    monkeypatch.chdir(tempfile.gettempdir())
+
+    result = pytester.runpytest("--pyargs", "--collect-only", "pkg.sub.test_it")
+    assert result.ret == ExitCode.OK
+    result.stdout.fnmatch_lines(
+        [
+            "<Package *pkg>",
+            "  <Package sub>",
+            "    <Module test_it.py>",
+            "      <Function test>",
+        ],
+        consecutive=True,
+    )
+
+
+def test_do_not_collect_symlink_siblings(
+    pytester: Pytester, tmp_path: Path, request: pytest.FixtureRequest
+) -> None:
+    """
+    Regression test for #12039: Do not collect from directories that are symlinks to other directories in the same path.
+
+    The check for short paths under Windows via os.path.samefile, introduced in #11936, also finds the symlinked
+    directory created by tmp_path/tmpdir.
+    """
+    # Use tmp_path because it creates a symlink with the name "current" next to the directory it creates.
+    symlink_path = tmp_path.parent / (tmp_path.name[:-1] + "current")
+    assert symlink_path.is_symlink() is True
+
+    # Create test file.
+    tmp_path.joinpath("test_foo.py").write_text("def test(): pass", encoding="UTF-8")
+
+    # Ensure we collect it only once if we pass the tmp_path.
+    result = pytester.runpytest(tmp_path, "-sv")
+    result.assert_outcomes(passed=1)
+
+    # Ensure we collect it only once if we pass the symlinked directory.
+    result = pytester.runpytest(symlink_path, "-sv")
+    result.assert_outcomes(passed=1)
+
+
+@pytest.mark.parametrize(
+    "exception_class, msg",
+    [
+        (KeyboardInterrupt, "*!!! KeyboardInterrupt !!!*"),
+        (SystemExit, "INTERNALERROR> SystemExit"),
+    ],
+)
+def test_respect_system_exceptions(
+    pytester: Pytester,
+    exception_class: type[BaseException],
+    msg: str,
+):
+    head = "Before exception"
+    tail = "After exception"
+    ensure_file(pytester.path / "test_eggs.py").write_text(
+        f"print('{head}')", encoding="UTF-8"
+    )
+    ensure_file(pytester.path / "test_ham.py").write_text(
+        f"raise {exception_class.__name__}()", encoding="UTF-8"
+    )
+    ensure_file(pytester.path / "test_spam.py").write_text(
+        f"print('{tail}')", encoding="UTF-8"
+    )
+
+    result = pytester.runpytest_subprocess("-s")
+    result.stdout.fnmatch_lines([f"*{head}*"])
+    result.stdout.fnmatch_lines([msg])
+    result.stdout.no_fnmatch_line(f"*{tail}*")
