@@ -1,10 +1,12 @@
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
 import enum
 from functools import cached_property
 from functools import partial
 from functools import wraps
 import sys
 from typing import TYPE_CHECKING
-from typing import Union
 
 from _pytest.compat import _PytestWrapper
 from _pytest.compat import assert_never
@@ -93,7 +95,7 @@ def test_get_real_func_partial() -> None:
     assert get_real_func(partial(foo)) is foo
 
 
-@pytest.mark.skipif(sys.version_info >= (3, 11), reason="couroutine removed")
+@pytest.mark.skipif(sys.version_info >= (3, 11), reason="coroutine removed")
 def test_is_generator_asyncio(pytester: Pytester) -> None:
     pytester.makepyfile(
         """
@@ -168,17 +170,17 @@ class ErrorsHelper:
 
 def test_helper_failures() -> None:
     helper = ErrorsHelper()
-    with pytest.raises(Exception):
-        helper.raise_exception
+    with pytest.raises(Exception):  # noqa: B017
+        _ = helper.raise_exception
     with pytest.raises(OutcomeException):
-        helper.raise_fail_outcome
+        _ = helper.raise_fail_outcome
 
 
 def test_safe_getattr() -> None:
     helper = ErrorsHelper()
     assert safe_getattr(helper, "raise_exception", "default") == "default"
     assert safe_getattr(helper, "raise_fail_outcome", "default") == "default"
-    with pytest.raises(BaseException):
+    with pytest.raises(BaseException):  # noqa: B017
         assert safe_getattr(helper, "raise_baseexception", "default")
 
 
@@ -215,7 +217,7 @@ def test_cached_property() -> None:
 
 
 def test_assert_never_union() -> None:
-    x: Union[int, str] = 10
+    x: int | str = 10
 
     if isinstance(x, int):
         pass
