@@ -1,4 +1,6 @@
 # mypy: allow-untyped-defs
+from __future__ import annotations
+
 import dataclasses
 import importlib.metadata
 import os
@@ -7,12 +9,7 @@ import re
 import sys
 import textwrap
 from typing import Any
-from typing import Dict
-from typing import List
 from typing import Sequence
-from typing import Tuple
-from typing import Type
-from typing import Union
 
 import _pytest._code
 from _pytest.config import _get_plugin_specs_as_list
@@ -67,13 +64,12 @@ class TestParseIni:
         p1 = pytester.makepyfile("def test(): pass")
         pytester.makefile(
             ".cfg",
-            setup="""
+            setup=f"""
                 [tool:pytest]
-                testpaths=%s
+                testpaths={p1.name}
                 [pytest]
                 testpaths=ignored
-        """
-            % p1.name,
+        """,
         )
         result = pytester.runpytest()
         result.stdout.fnmatch_lines(["configfile: setup.cfg", "* 1 passed in *"])
@@ -634,13 +630,13 @@ class TestConfigCmdlineParsing:
 class TestConfigAPI:
     def test_config_trace(self, pytester: Pytester) -> None:
         config = pytester.parseconfig()
-        values: List[str] = []
+        values: list[str] = []
         config.trace.root.setwriter(values.append)
         config.trace("hello")
         assert len(values) == 1
         assert values[0] == "hello [config]\n"
 
-    def test_config_getoption(self, pytester: Pytester) -> None:
+    def test_config_getoption_declared_option_name(self, pytester: Pytester) -> None:
         pytester.makeconftest(
             """
             def pytest_addoption(parser):
@@ -652,6 +648,18 @@ class TestConfigAPI:
             assert config.getoption(x) == "this"
         pytest.raises(ValueError, config.getoption, "qweqwe")
 
+        config_novalue = pytester.parseconfig()
+        assert config_novalue.getoption("hello") is None
+        assert config_novalue.getoption("hello", default=1) is None
+        assert config_novalue.getoption("hello", default=1, skip=True) == 1
+
+    def test_config_getoption_undeclared_option_name(self, pytester: Pytester) -> None:
+        config = pytester.parseconfig()
+        with pytest.raises(ValueError):
+            config.getoption("x")
+        assert config.getoption("x", default=1) == 1
+        assert config.getoption("x", default=1, skip=True) == 1
+
     def test_config_getoption_unicode(self, pytester: Pytester) -> None:
         pytester.makeconftest(
             """
@@ -679,12 +687,6 @@ class TestConfigAPI:
         with pytest.raises(pytest.skip.Exception):
             config.getvalueorskip("hello")
 
-    def test_getoption(self, pytester: Pytester) -> None:
-        config = pytester.parseconfig()
-        with pytest.raises(ValueError):
-            config.getvalue("x")
-        assert config.getoption("x", 1) == 1
-
     def test_getconftest_pathlist(self, pytester: Pytester, tmp_path: Path) -> None:
         somepath = tmp_path.joinpath("x", "y", "z")
         p = tmp_path.joinpath("conftest.py")
@@ -838,11 +840,10 @@ class TestConfigAPI:
         )
         if str_val != "no-ini":
             pytester.makeini(
-                """
+                f"""
                 [pytest]
-                strip=%s
+                strip={str_val}
             """
-                % str_val
             )
         config = pytester.parseconfig()
         assert config.getini("strip") is bool_val
@@ -922,7 +923,7 @@ class TestConfigAPI:
         # default for string is ""
         value = config.getini("string1")
         assert value == ""
-        # should return None if None is explicity set as default value
+        # should return None if None is explicitly set as default value
         # irrespective of the type argument
         value = config.getini("none_1")
         assert value is None
@@ -998,7 +999,7 @@ class TestConfigFromdictargs:
 
     def test_invocation_params_args(self, _sys_snapshot) -> None:
         """Show that fromdictargs can handle args in their "orig" format"""
-        option_dict: Dict[str, object] = {}
+        option_dict: dict[str, object] = {}
         args = ["-vvvv", "-s", "a", "b"]
 
         config = Config.fromdictargs(option_dict, args)
@@ -1212,7 +1213,7 @@ def test_plugin_preparse_prevents_setuptools_loading(
 def test_disable_plugin_autoload(
     pytester: Pytester,
     monkeypatch: MonkeyPatch,
-    parse_args: Union[Tuple[str, str], Tuple[()]],
+    parse_args: tuple[str, str] | tuple[()],
     should_load: bool,
 ) -> None:
     class DummyEntryPoint:
@@ -1290,8 +1291,8 @@ def test_invalid_options_show_extra_information(pytester: Pytester) -> None:
     result.stderr.fnmatch_lines(
         [
             "*error: unrecognized arguments: --invalid-option*",
-            "*  inifile: %s*" % pytester.path.joinpath("tox.ini"),
-            "*  rootdir: %s*" % pytester.path,
+            "*  inifile: {}*".format(pytester.path.joinpath("tox.ini")),
+            f"*  rootdir: {pytester.path}*",
         ]
     )
 
@@ -1306,7 +1307,7 @@ def test_invalid_options_show_extra_information(pytester: Pytester) -> None:
     ],
 )
 def test_consider_args_after_options_for_rootdir(
-    pytester: Pytester, args: List[str]
+    pytester: Pytester, args: list[str]
 ) -> None:
     """
     Consider all arguments in the command-line for rootdir
@@ -1423,8 +1424,8 @@ def test_load_initial_conftest_last_ordering(_config_for_test):
 def test_get_plugin_specs_as_list() -> None:
     def exp_match(val: object) -> str:
         return (
-            "Plugins may be specified as a sequence or a ','-separated string of plugin names. Got: %s"
-            % re.escape(repr(val))
+            f"Plugins may be specified as a sequence or a ','-separated string "
+            f"of plugin names. Got: {re.escape(repr(val))}"
         )
 
     with pytest.raises(pytest.UsageError, match=exp_match({"foo"})):
@@ -1837,10 +1838,10 @@ class TestOverrideIniArgs:
         self, monkeypatch: MonkeyPatch, _config_for_test, _sys_snapshot
     ) -> None:
         cache_dir = ".custom_cache"
-        monkeypatch.setenv("PYTEST_ADDOPTS", "-o cache_dir=%s" % cache_dir)
+        monkeypatch.setenv("PYTEST_ADDOPTS", f"-o cache_dir={cache_dir}")
         config = _config_for_test
         config._preparse([], addopts=True)
-        assert config._override_ini == ["cache_dir=%s" % cache_dir]
+        assert config._override_ini == [f"cache_dir={cache_dir}"]
 
     def test_addopts_from_env_not_concatenated(
         self, monkeypatch: MonkeyPatch, _config_for_test
@@ -2048,7 +2049,7 @@ def test_invocation_args(pytester: Pytester) -> None:
 )
 def test_config_blocked_default_plugins(pytester: Pytester, plugin: str) -> None:
     p = pytester.makepyfile("def test(): pass")
-    result = pytester.runpytest(str(p), "-pno:%s" % plugin)
+    result = pytester.runpytest(str(p), f"-pno:{plugin}")
 
     if plugin == "python":
         assert result.ret == ExitCode.USAGE_ERROR
@@ -2065,7 +2066,7 @@ def test_config_blocked_default_plugins(pytester: Pytester, plugin: str) -> None
         result.stdout.fnmatch_lines(["* 1 passed in *"])
 
     p = pytester.makepyfile("def test(): assert 0")
-    result = pytester.runpytest(str(p), "-pno:%s" % plugin)
+    result = pytester.runpytest(str(p), f"-pno:{plugin}")
     assert result.ret == ExitCode.TESTS_FAILED
     if plugin != "terminal":
         result.stdout.fnmatch_lines(["* 1 failed in *"])
@@ -2243,7 +2244,7 @@ def test_strtobool() -> None:
     ],
 )
 def test_parse_warning_filter(
-    arg: str, escape: bool, expected: Tuple[str, str, Type[Warning], str, int]
+    arg: str, escape: bool, expected: tuple[str, str, type[Warning], str, int]
 ) -> None:
     assert parse_warning_filter(arg, escape=escape) == expected
 
