@@ -1,14 +1,13 @@
 # mypy: allow-untyped-defs
+from __future__ import annotations
+
 from datetime import datetime
+from datetime import timezone
 import os
 from pathlib import Path
 import platform
 from typing import cast
-from typing import List
-from typing import Optional
-from typing import Tuple
 from typing import TYPE_CHECKING
-from typing import Union
 from xml.dom import minidom
 
 import xmlschema
@@ -39,12 +38,12 @@ class RunAndParse:
         self.schema = schema
 
     def __call__(
-        self, *args: Union[str, "os.PathLike[str]"], family: Optional[str] = "xunit1"
-    ) -> Tuple[RunResult, "DomNode"]:
+        self, *args: str | os.PathLike[str], family: str | None = "xunit1"
+    ) -> tuple[RunResult, DomNode]:
         if family:
             args = ("-o", "junit_family=" + family, *args)
         xml_path = self.pytester.path.joinpath("junit.xml")
-        result = self.pytester.runpytest("--junitxml=%s" % xml_path, *args)
+        result = self.pytester.runpytest(f"--junitxml={xml_path}", *args)
         if family == "xunit2":
             with xml_path.open(encoding="utf-8") as f:
                 self.schema.validate(f)
@@ -220,11 +219,11 @@ class TestPython:
                 pass
         """
         )
-        start_time = datetime.now()
+        start_time = datetime.now(timezone.utc)
         result, dom = run_and_parse(family=xunit_family)
         node = dom.find_first_by_tag("testsuite")
-        timestamp = datetime.strptime(node["timestamp"], "%Y-%m-%dT%H:%M:%S.%f")
-        assert start_time <= timestamp < datetime.now()
+        timestamp = datetime.strptime(node["timestamp"], "%Y-%m-%dT%H:%M:%S.%f%z")
+        assert start_time <= timestamp < datetime.now(timezone.utc)
 
     def test_timing_function(
         self, pytester: Pytester, run_and_parse: RunAndParse, mock_timing
@@ -520,7 +519,7 @@ class TestPython:
         )
 
         result, dom = run_and_parse(
-            "-o", "junit_logging=%s" % junit_logging, family=xunit_family
+            "-o", f"junit_logging={junit_logging}", family=xunit_family
         )
         assert result.ret, "Expected ret > 0"
         node = dom.find_first_by_tag("testsuite")
@@ -605,11 +604,11 @@ class TestPython:
         for index, char in enumerate("<&'"):
             tnode = node.find_nth_by_tag("testcase", index)
             tnode.assert_attr(
-                classname="test_failure_escape", name="test_func[%s]" % char
+                classname="test_failure_escape", name=f"test_func[{char}]"
             )
             sysout = tnode.find_first_by_tag("system-out")
             text = sysout.text
-            assert "%s\n" % char in text
+            assert f"{char}\n" in text
 
     @parametrize_families
     def test_junit_prefixing(
@@ -694,7 +693,7 @@ class TestPython:
                 assert 0
         """
         )
-        result, dom = run_and_parse("-o", "junit_logging=%s" % junit_logging)
+        result, dom = run_and_parse("-o", f"junit_logging={junit_logging}")
         node = dom.find_first_by_tag("testsuite")
         tnode = node.find_first_by_tag("testcase")
         if junit_logging in ["system-err", "out-err", "all"]:
@@ -764,13 +763,12 @@ class TestPython:
     def test_unicode(self, pytester: Pytester, run_and_parse: RunAndParse) -> None:
         value = "hx\xc4\x85\xc4\x87\n"
         pytester.makepyfile(
-            """\
+            f"""\
             # coding: latin1
             def test_hello():
-                print(%r)
+                print({value!r})
                 assert 0
             """
-            % value
         )
         result, dom = run_and_parse()
         assert result.ret == 1
@@ -805,7 +803,7 @@ class TestPython:
                 print('hello-stdout')
         """
         )
-        result, dom = run_and_parse("-o", "junit_logging=%s" % junit_logging)
+        result, dom = run_and_parse("-o", f"junit_logging={junit_logging}")
         node = dom.find_first_by_tag("testsuite")
         pnode = node.find_first_by_tag("testcase")
         if junit_logging == "no":
@@ -829,7 +827,7 @@ class TestPython:
                 sys.stderr.write('hello-stderr')
         """
         )
-        result, dom = run_and_parse("-o", "junit_logging=%s" % junit_logging)
+        result, dom = run_and_parse("-o", f"junit_logging={junit_logging}")
         node = dom.find_first_by_tag("testsuite")
         pnode = node.find_first_by_tag("testcase")
         if junit_logging == "no":
@@ -858,7 +856,7 @@ class TestPython:
                 pass
         """
         )
-        result, dom = run_and_parse("-o", "junit_logging=%s" % junit_logging)
+        result, dom = run_and_parse("-o", f"junit_logging={junit_logging}")
         node = dom.find_first_by_tag("testsuite")
         pnode = node.find_first_by_tag("testcase")
         if junit_logging == "no":
@@ -888,7 +886,7 @@ class TestPython:
                 pass
         """
         )
-        result, dom = run_and_parse("-o", "junit_logging=%s" % junit_logging)
+        result, dom = run_and_parse("-o", f"junit_logging={junit_logging}")
         node = dom.find_first_by_tag("testsuite")
         pnode = node.find_first_by_tag("testcase")
         if junit_logging == "no":
@@ -919,7 +917,7 @@ class TestPython:
                 sys.stdout.write('hello-stdout call')
         """
         )
-        result, dom = run_and_parse("-o", "junit_logging=%s" % junit_logging)
+        result, dom = run_and_parse("-o", f"junit_logging={junit_logging}")
         node = dom.find_first_by_tag("testsuite")
         pnode = node.find_first_by_tag("testcase")
         if junit_logging == "no":
@@ -941,7 +939,7 @@ def test_mangle_test_address() -> None:
 
 
 def test_dont_configure_on_workers(tmp_path: Path) -> None:
-    gotten: List[object] = []
+    gotten: list[object] = []
 
     class FakeConfig:
         if TYPE_CHECKING:
@@ -1002,7 +1000,7 @@ class TestNonPython:
 
 @pytest.mark.parametrize("junit_logging", ["no", "system-out"])
 def test_nullbyte(pytester: Pytester, junit_logging: str) -> None:
-    # A null byte can not occur in XML (see section 2.2 of the spec)
+    # A null byte cannot occur in XML (see section 2.2 of the spec)
     pytester.makepyfile(
         """
         import sys
@@ -1013,7 +1011,7 @@ def test_nullbyte(pytester: Pytester, junit_logging: str) -> None:
     """
     )
     xmlf = pytester.path.joinpath("junit.xml")
-    pytester.runpytest("--junitxml=%s" % xmlf, "-o", "junit_logging=%s" % junit_logging)
+    pytester.runpytest(f"--junitxml={xmlf}", "-o", f"junit_logging={junit_logging}")
     text = xmlf.read_text(encoding="utf-8")
     assert "\x00" not in text
     if junit_logging == "system-out":
@@ -1035,7 +1033,7 @@ def test_nullbyte_replace(pytester: Pytester, junit_logging: str) -> None:
     """
     )
     xmlf = pytester.path.joinpath("junit.xml")
-    pytester.runpytest("--junitxml=%s" % xmlf, "-o", "junit_logging=%s" % junit_logging)
+    pytester.runpytest(f"--junitxml={xmlf}", "-o", f"junit_logging={junit_logging}")
     text = xmlf.read_text(encoding="utf-8")
     if junit_logging == "system-out":
         assert "#x0" in text
@@ -1071,9 +1069,9 @@ def test_invalid_xml_escape() -> None:
     for i in invalid:
         got = bin_xml_escape(chr(i))
         if i <= 0xFF:
-            expected = "#x%02X" % i
+            expected = f"#x{i:02X}"
         else:
-            expected = "#x%04X" % i
+            expected = f"#x{i:04X}"
         assert got == expected
     for i in valid:
         assert chr(i) == bin_xml_escape(chr(i))
@@ -1184,7 +1182,7 @@ def test_unicode_issue368(pytester: Pytester) -> None:
 
     class Report(BaseReport):
         longrepr = ustr
-        sections: List[Tuple[str, str]] = []
+        sections: list[tuple[str, str]] = []
         nodeid = "something"
         location = "tests/filename.py", 42, "TestClass.method"
         when = "teardown"
@@ -1202,7 +1200,7 @@ def test_unicode_issue368(pytester: Pytester) -> None:
     node_reporter.append_skipped(test_report)
     test_report.longrepr = "filename", 1, "Skipped: 卡嘣嘣"
     node_reporter.append_skipped(test_report)
-    test_report.wasxfail = ustr  # type: ignore[attr-defined]
+    test_report.wasxfail = ustr
     node_reporter.append_skipped(test_report)
     log.pytest_sessionfinish()
 
@@ -1496,7 +1494,7 @@ def test_global_properties(pytester: Pytester, xunit_family: str) -> None:
     log = LogXML(str(path), None, family=xunit_family)
 
     class Report(BaseReport):
-        sections: List[Tuple[str, str]] = []
+        sections: list[tuple[str, str]] = []
         nodeid = "test_node_id"
 
     log.pytest_sessionstart()
@@ -1532,7 +1530,7 @@ def test_url_property(pytester: Pytester) -> None:
 
     class Report(BaseReport):
         longrepr = "FooBarBaz"
-        sections: List[Tuple[str, str]] = []
+        sections: list[tuple[str, str]] = []
         nodeid = "something"
         location = "tests/filename.py", 42, "TestClass.method"
         url = test_url
@@ -1748,7 +1746,7 @@ def test_logging_passing_tests_disabled_logs_output_for_failing_test_issue5430(
     """
     )
     result, dom = run_and_parse(
-        "-o", "junit_logging=%s" % junit_logging, family=xunit_family
+        "-o", f"junit_logging={junit_logging}", family=xunit_family
     )
     assert result.ret == 1
     node = dom.find_first_by_tag("testcase")
