@@ -1,7 +1,8 @@
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
 import os
 import sys
-from typing import List
-from typing import Optional
 from unittest import mock
 
 from _pytest.config import ExitCode
@@ -17,7 +18,7 @@ class TestMark:
     @pytest.mark.parametrize("attr", ["mark", "param"])
     def test_pytest_exists_in_namespace_all(self, attr: str) -> None:
         module = sys.modules["pytest"]
-        assert attr in module.__all__  # type: ignore
+        assert attr in module.__all__
 
     def test_pytest_mark_notcallable(self) -> None:
         mark = MarkGenerator(_ispytest=True)
@@ -33,7 +34,7 @@ class TestMark:
 
         assert pytest.mark.foo(some_function) is some_function
         marked_with_args = pytest.mark.foo.with_args(some_function)
-        assert marked_with_args is not some_function  # type: ignore[comparison-overlap]
+        assert marked_with_args is not some_function
 
         assert pytest.mark.foo(SomeClass) is SomeClass
         assert pytest.mark.foo.with_args(SomeClass) is not SomeClass  # type: ignore[comparison-overlap]
@@ -41,7 +42,7 @@ class TestMark:
     def test_pytest_mark_name_starts_with_underscore(self) -> None:
         mark = MarkGenerator(_ispytest=True)
         with pytest.raises(AttributeError):
-            mark._some_name
+            _ = mark._some_name
 
 
 def test_marked_class_run_twice(pytester: Pytester) -> None:
@@ -213,7 +214,7 @@ def test_strict_prohibits_unregistered_markers(
     ],
 )
 def test_mark_option(
-    expr: str, expected_passed: List[Optional[str]], pytester: Pytester
+    expr: str, expected_passed: list[str | None], pytester: Pytester
 ) -> None:
     pytester.makepyfile(
         """
@@ -232,12 +233,60 @@ def test_mark_option(
     assert passed_str == expected_passed
 
 
+@pytest.mark.parametrize(
+    ("expr", "expected_passed"),
+    [
+        ("car(color='red')", ["test_one"]),
+        ("car(color='red') or car(color='blue')", ["test_one", "test_two"]),
+        ("car and not car(temp=5)", ["test_one", "test_three"]),
+        ("car(temp=4)", ["test_one"]),
+        ("car(temp=4) or car(temp=5)", ["test_one", "test_two"]),
+        ("car(temp=4) and car(temp=5)", []),
+        ("car(temp=-5)", ["test_three"]),
+        ("car(ac=True)", ["test_one"]),
+        ("car(ac=False)", ["test_two"]),
+        ("car(ac=None)", ["test_three"]),  # test NOT_NONE_SENTINEL
+    ],
+    ids=str,
+)
+def test_mark_option_with_kwargs(
+    expr: str, expected_passed: list[str | None], pytester: Pytester
+) -> None:
+    pytester.makepyfile(
+        """
+        import pytest
+        @pytest.mark.car
+        @pytest.mark.car(ac=True)
+        @pytest.mark.car(temp=4)
+        @pytest.mark.car(color="red")
+        def test_one():
+            pass
+        @pytest.mark.car
+        @pytest.mark.car(ac=False)
+        @pytest.mark.car(temp=5)
+        @pytest.mark.car(color="blue")
+        def test_two():
+            pass
+        @pytest.mark.car
+        @pytest.mark.car(ac=None)
+        @pytest.mark.car(temp=-5)
+        def test_three():
+            pass
+
+    """
+    )
+    rec = pytester.inline_run("-m", expr)
+    passed, skipped, fail = rec.listoutcomes()
+    passed_str = [x.nodeid.split("::")[-1] for x in passed]
+    assert passed_str == expected_passed
+
+
 @pytest.mark.parametrize(
     ("expr", "expected_passed"),
     [("interface", ["test_interface"]), ("not interface", ["test_nointer"])],
 )
 def test_mark_option_custom(
-    expr: str, expected_passed: List[str], pytester: Pytester
+    expr: str, expected_passed: list[str], pytester: Pytester
 ) -> None:
     pytester.makeconftest(
         """
@@ -275,7 +324,7 @@ def test_mark_option_custom(
     ],
 )
 def test_keyword_option_custom(
-    expr: str, expected_passed: List[str], pytester: Pytester
+    expr: str, expected_passed: list[str], pytester: Pytester
 ) -> None:
     pytester.makepyfile(
         """
@@ -313,7 +362,7 @@ def test_keyword_option_considers_mark(pytester: Pytester) -> None:
     ],
 )
 def test_keyword_option_parametrize(
-    expr: str, expected_passed: List[str], pytester: Pytester
+    expr: str, expected_passed: list[str], pytester: Pytester
 ) -> None:
     pytester.makepyfile(
         """
@@ -371,6 +420,10 @@ def test_parametrize_with_module(pytester: Pytester) -> None:
             "not or",
             "at column 5: expected not OR left parenthesis OR identifier; got or",
         ),
+        (
+            "nonexistent_mark(non_supported='kwarg')",
+            "Keyword expressions do not support call parameters",
+        ),
     ],
 )
 def test_keyword_option_wrong_arguments(
@@ -894,7 +947,7 @@ class TestKeywordSelection:
         )
         monkeypatch.chdir(pytester.path / "suite")
 
-        def get_collected_names(*args: str) -> List[str]:
+        def get_collected_names(*args: str) -> list[str]:
             _, rec = pytester.inline_genitems(*args)
             calls = rec.getcalls("pytest_collection_finish")
             assert len(calls) == 1
@@ -929,7 +982,7 @@ class TestMarkDecorator:
 
 @pytest.mark.parametrize("mark", [None, "", "skip", "xfail"])
 def test_parameterset_for_parametrize_marks(
-    pytester: Pytester, mark: Optional[str]
+    pytester: Pytester, mark: str | None
 ) -> None:
     if mark is not None:
         pytester.makeini(
