@@ -1,8 +1,9 @@
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
 import sys
 from textwrap import dedent
 from typing import Generator
-from typing import List
-from typing import Optional
 
 from _pytest.pytester import Pytester
 import pytest
@@ -90,8 +91,8 @@ def test_clean_up(pytester: Pytester) -> None:
     pytester.makefile(".ini", pytest="[pytest]\npythonpath=I_SHALL_BE_REMOVED\n")
     pytester.makepyfile(test_foo="""def test_foo(): pass""")
 
-    before: Optional[List[str]] = None
-    after: Optional[List[str]] = None
+    before: list[str] | None = None
+    after: list[str] | None = None
 
     class Plugin:
         @pytest.hookimpl(wrapper=True, tryfirst=True)
