@@ -1,4 +1,6 @@
 # mypy: allow-untyped-defs
+from __future__ import annotations
+
 import sys
 import textwrap
 
@@ -297,13 +299,12 @@ class TestXFail:
     @pytest.mark.parametrize("strict", [True, False])
     def test_xfail_simple(self, pytester: Pytester, strict: bool) -> None:
         item = pytester.getitem(
-            """
+            f"""
             import pytest
-            @pytest.mark.xfail(strict=%s)
+            @pytest.mark.xfail(strict={strict})
             def test_func():
                 assert 0
         """
-            % strict
         )
         reports = runtestprotocol(item, log=False)
         assert len(reports) == 3
@@ -630,15 +631,14 @@ class TestXFail:
     @pytest.mark.parametrize("strict", [True, False])
     def test_strict_xfail(self, pytester: Pytester, strict: bool) -> None:
         p = pytester.makepyfile(
-            """
+            f"""
             import pytest
 
-            @pytest.mark.xfail(reason='unsupported feature', strict=%s)
+            @pytest.mark.xfail(reason='unsupported feature', strict={strict})
             def test_foo():
                 with open('foo_executed', 'w', encoding='utf-8'):
                     pass  # make sure test executes
         """
-            % strict
         )
         result = pytester.runpytest(p, "-rxX")
         if strict:
@@ -658,14 +658,13 @@ class TestXFail:
     @pytest.mark.parametrize("strict", [True, False])
     def test_strict_xfail_condition(self, pytester: Pytester, strict: bool) -> None:
         p = pytester.makepyfile(
-            """
+            f"""
             import pytest
 
-            @pytest.mark.xfail(False, reason='unsupported feature', strict=%s)
+            @pytest.mark.xfail(False, reason='unsupported feature', strict={strict})
             def test_foo():
                 pass
         """
-            % strict
         )
         result = pytester.runpytest(p, "-rxX")
         result.stdout.fnmatch_lines(["*1 passed*"])
@@ -674,14 +673,13 @@ class TestXFail:
     @pytest.mark.parametrize("strict", [True, False])
     def test_xfail_condition_keyword(self, pytester: Pytester, strict: bool) -> None:
         p = pytester.makepyfile(
-            """
+            f"""
             import pytest
 
-            @pytest.mark.xfail(condition=False, reason='unsupported feature', strict=%s)
+            @pytest.mark.xfail(condition=False, reason='unsupported feature', strict={strict})
             def test_foo():
                 pass
         """
-            % strict
         )
         result = pytester.runpytest(p, "-rxX")
         result.stdout.fnmatch_lines(["*1 passed*"])
@@ -692,11 +690,10 @@ class TestXFail:
         self, pytester: Pytester, strict_val
     ) -> None:
         pytester.makeini(
-            """
+            f"""
             [pytest]
-            xfail_strict = %s
+            xfail_strict = {strict_val}
         """
-            % strict_val
         )
         p = pytester.makepyfile(
             """
@@ -1143,10 +1140,10 @@ def test_errors_in_xfail_skip_expressions(pytester: Pytester) -> None:
     result = pytester.runpytest()
     markline = "            ^"
     pypy_version_info = getattr(sys, "pypy_version_info", None)
-    if pypy_version_info is not None and pypy_version_info < (6,):
-        markline = markline[1:]
+    if pypy_version_info is not None:
+        markline = markline[7:]
 
-    if sys.version_info[:2] >= (3, 10):
+    if sys.version_info >= (3, 10):
         expected = [
             "*ERROR*test_nameerror*",
             "*asd*",
